#include <stdio.h>
#include <zephyr.h>
#include <device.h>
#include <drivers/sensor.h>
#include <drivers/gpio.h>
#include <drivers/i2c.h>
#include <kernel.h>
#include <sys/printk.h>
#include "as6212.h"
float as6212_gett_high(uint8_t sensorId);

float last_read_tempc;
float get_last_read_temp() 
{
 return last_read_tempc; 
}

uint16_t as6212_read_register(uint8_t sensorId, uint8_t reg);
void as6212_write_register(uint8_t sensorId, uint8_t reg, int16_t data);
static int32_t as6212_write(uint8_t sensorId, uint8_t reg, const uint8_t *bufp,
                              uint16_t len);
static int32_t as6212_read(uint8_t sensorId, uint8_t reg, uint8_t *bufp,
                             uint16_t len);
static void as6212_delay(uint32_t ms);

extern struct device *i2c_dev;
extern struct device *i2c_dev2;
void as6212_read_data_polling(void *p1, void *p2, void *p3);

float __hts_read_loop(uint8_t sensorId);

#define AS6212_MONITOR_STACK_SIZE 2048
#define AS6212_MONITOR_THREAD_PRIORITY 5
K_THREAD_STACK_DEFINE(as6212_monitor_thread_stack, AS6212_MONITOR_STACK_SIZE);
struct k_thread as6212_monitor_thread_data;
static K_FIFO_DEFINE(as6212_monitor_queue);


void create_as6212_monitoring_thread() 
{
  k_tid_t my_tid = k_thread_create(&as6212_monitor_thread_data,
      as6212_monitor_thread_stack,
      K_THREAD_STACK_SIZEOF(as6212_monitor_thread_stack),
      as6212_read_data_polling,
      NULL,
      NULL,
      NULL,
      AS6212_MONITOR_THREAD_PRIORITY,
      0,
      K_NO_WAIT);
  k_thread_name_set(&as6212_monitor_thread_data, "as6212 Monitoring Thread");
}

void as6212_read_data_polling(void *p1, void *p2, void *p3)
{

    while(1)
    {
            
        printk("temp sensor1  %f \r\n", __hts_read_loop(0));
        printk("temp sensor2  %f \r\n", __hts_read_loop(1));
        k_msleep(1000);
    }
}


static int32_t as6212_write(uint8_t sensorId, uint8_t reg, const uint8_t *bufp,
                              uint16_t len)
{
   if (sensorId == 0)
     i2c_burst_write(i2c_dev, 0x4A, reg, bufp, len);
   else
     i2c_burst_write(i2c_dev2, 0x4A, reg, bufp, len);

   return 0;
}

static int32_t as6212_read(uint8_t sensorId, uint8_t reg, uint8_t *bufp,
                             uint16_t len)
{
   if (sensorId == 0)
     i2c_burst_read(i2c_dev, 0x4A, reg, bufp, len);
   else
     i2c_burst_read(i2c_dev2, 0x4A, reg, bufp, len);
   
   return 0;
}


uint16_t as6212_read_register(uint8_t sensorId, uint8_t reg)
{
  uint8_t data_buffer[2]; 
  int16_t datac = 0;
  as6212_read(sensorId, reg, data_buffer, 2);
  datac = ((data_buffer[0] << 8) | data_buffer[1]);
  return datac;
}

void as6212_write_register(uint8_t sensorId, uint8_t reg, int16_t data)
{
  as6212_write(sensorId, reg, (uint8_t *)&data, 2);
}

bool is_new_hts_data_available= false;

bool __is_new_data_available_for_hts_send_loop(){
  return is_new_hts_data_available; 
}

void __reset_is_new_hts_data_available(){
  is_new_hts_data_available= false;
}

float __hts_read_loop(uint8_t sensorId)
{
  volatile int16_t digital_tempc = as6212_read_register(sensorId, TVAL);
  float final_tempc;
  if(digital_tempc < 32768)
  {
    final_tempc = digital_tempc * 0.0078125;
  }
  if(digital_tempc >= 32768)
  {
    final_tempc = ((digital_tempc - 1) * 0.0078125) * -1;
  }

  last_read_tempc= final_tempc;	
  is_new_hts_data_available= true;

  return last_read_tempc;
}

float as6212_read_tempf(uint8_t sensorId)
{
  return as6212_read_tempc(sensorId) * 9.0 / 5.0 + 32.0;
}

float as6212_gett_low(uint8_t sensorId)
{
  int16_t low_temp = as6212_read_register(sensorId, TLOW);
  float temp;
  if (low_temp < 32768)
  {
    temp = low_temp * 0.0078125;
  }
  if (low_temp >= 32768)
  {
    temp = ((low_temp - 1) * 0.0078125) * -1;
  }
  return temp;
}

/*
 * sets tlow threshold, if temp drops below the threshold then 
 * interrupt is triggered. pointer to error flag to for custom
 * error reporting.
 */
bool as6212_sett_low(uint8_t sensorId, int16_t low_limit, uint8_t *tlow_err_flag)
{        
  if (low_limit < as6212_gett_high(sensorId) && low_limit != as6212_gett_low(sensorId))
  {
    int16_t low_temp = low_limit / 0.0078125;
    as6212_write_register(sensorId, TLOW, low_temp);
    return true;
  }
  
  else if(low_limit < as6212_gett_high(sensorId) && low_limit == as6212_gett_low(sensorId))
  {    
    *tlow_err_flag = 1;
    return false;
  } 
  else
  {  
    *tlow_err_flag = 2;
    return false;
  }
}

float as6212_gett_high(uint8_t sensorId)
{
  int16_t high_temp = as6212_read_register(sensorId, THIGH);
  float temp;
  if(high_temp < 32768)
  {
          temp = high_temp * 0.0078125;
  }
  if(high_temp >= 32768)
  {
          temp = ((high_temp - 1) * 0.0078125) * -1;
  }
  return temp;
}

bool as6212_sett_high(uint8_t sensorId , int16_t high_limit)
{
  if(high_limit > as6212_gett_low(sensorId))
  {
    int16_t high_temp = high_limit / 0.0078125;
    as6212_write_register(sensorId, THIGH, high_temp);
    return true;
  }
  else
  {
    //serial.println("value is below the low temperature threshold.\n please choose a different value.");
   return false;
  }
}

uint16_t as6212_read_config(uint8_t sensorId)
{
  return as6212_read_register(sensorId, CONFIG);
}

void as6212_set_config(uint8_t sensorId, uint16_t target_state)
{		
  as6212_write_register(sensorId, CONFIG, target_state);		
}

//sleep single-shot mode (0xc1a0) returns odd register value (ffffc1a0)

